/*********************************************************************************
 * Copyright (c) 2006 Forschungszentrum Juelich GmbH 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Forschungszentrum Juelich GmbH nor the names of its 
 * contributors may be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *********************************************************************************/


package de.fzj.unicore.xuudb.client;

import java.io.File;

import de.fzj.unicore.xuudb.Configuration;

public class XUUDBClient {


	private static String configFile;

	public static void main(String[] args) {
		configFile=System.getProperty("xuudb.client.conf",new File("conf","xuudb_client.conf").getAbsolutePath());
		if(args.length==0)
			printUsage();
		else if (args[0].equalsIgnoreCase("--help"))
			printUsage();
		else
			XUUDBClient.parseArgs(args);
	}

	private static void parseArgs(String[] args) {
		if (args[0].equalsIgnoreCase("--init")) {
			Configuration.createDefaultClient();
		} else {
			Boolean hasRealArgs = true;
			if (args[0].equalsIgnoreCase("--config") && args.length>1) {
				configFile=args[1];
				if (args.length>2) {
					String[] shiftedArgs = new String[args.length-2];
					System.arraycopy(args, 2, shiftedArgs, 0, args.length-2);
					args = shiftedArgs;
				} else {
					hasRealArgs = false; 
				}
			}

			if (hasRealArgs == true) {
				try {
					AdminController ac=new AdminController(configFile);
					ac.run(args);
				} catch (Exception e) {
					e.printStackTrace();
					System.out.println("Cannot perform admintask. BYE.");
					System.exit(-1);
				}
			} else {
				printUsage();
			}	

		}
	}


	public static void printUsage() {
		System.out.println(
				"\n" +
				"Usage: admin  <command>  [parameters]\n" +
				"Commands are:\n" +
				"   add         <gcID>  <pemfile>  <xlogin> <role> [project1[,project2[,...]]]\n" +
				"   adddn       <gcID>  <DN>  <xlogin> <role> [project1[,project2[,...]]]\n" +
				"   remove      ALL|gcID=x|pemfile=file|dn=\"DN\"|role=x|xlogin=x|project=x\n" +
				"   list        gcID=x|pemfile=file|dn=\"DN\"|role=x|xlogin=x|project=x\n" +
				"   update      <gcID> <pemfile or DN> gcID=x|pemfile=file|dn=\"dn\"|role=x|xlogin=x|project=x\n" +
				"   export      csv-file [force]\n" +
				"   import      csv-file [clearDB]\n" +
				"   check-cert  gcid pemfile \n" +
				"   check-dn    gcid dn \n" +
				"   --init      creates a new admin client configuration in ./conf if not existing\n" +
		"   help -h 	print this help");
	}
}
