package de.fzj.unicore.xuudb.client.functional;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.PrintStream;
import java.security.Permission;
import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;
import de.fzj.unicore.xuudb.Configuration;
import de.fzj.unicore.xuudb.client.XUUDBClient;
import de.fzj.unicore.xuudb.server.HttpsServer;

public abstract class TestBase extends TestCase {
	protected ByteArrayOutputStream out;
	protected PrintStream Sout;
	protected PrintStream Serr;
	private SecurityManager man;
	
	
	//Universal conf to server and client
	private String configFile = "src/test/resources/universal.conf"; 
	protected static boolean haveInit = false;

	protected static class ExitException extends SecurityException {
		private static final long serialVersionUID = 1L;
		public final int status;

		public ExitException(int status) {
			this.status = status;
		}
	}

	private static class NoExitSecurityManager extends SecurityManager {
		public void checkPermission(Permission perm) {
			// allow anything.
		}

		public void checkPermission(Permission perm, Object context) {
			// allow anything.
		}

		@Override
		public void checkExit(int status) {
			super.checkExit(status);
			throw new ExitException(status);

		}
	}

	protected void setUp() throws Exception {

		if (haveInit) {
			man=System.getSecurityManager();
			System.setSecurityManager(new NoExitSecurityManager());
			out = new ByteArrayOutputStream();
			Sout=System.out;
			Serr=System.err;
			
			System.setOut(new PrintStream(out));
			System.setErr(new PrintStream(out));
		} else {
			
			Configuration conf=Configuration.createConfiguration(new File(configFile));
			conf.setProperty("xuudb_data_file", System.getProperty("java.io.tmpdir")+File.separator+"xuudb_test_"+System.currentTimeMillis());
			new HttpsServer(conf).start();
			man=System.getSecurityManager();
			System.setSecurityManager(new NoExitSecurityManager());
			out = new ByteArrayOutputStream();
			Sout=System.out;
			Serr=System.err;
			System.setOut(new PrintStream(out));
			System.setErr(new PrintStream(out));
			haveInit = true;
		}

	}

	protected void tearDown() throws Exception {
		System.setSecurityManager(man); // or save and restore original
		System.setOut(Sout);
		System.setErr(Serr);
	}

	protected void runClient(List<String> args) {

		List<String> arg = new ArrayList<String>();
		arg.add("--config");
		arg.add(configFile);
		arg.addAll(args);
		String[] a = new String[arg.size()];
		arg.toArray(a);
		for (int i = 0; i < a.length; i++)
			System.out.print(a[i] + " ");
		XUUDBClient.main(a);

	}

	protected boolean searchInOut(String searchString) {
		return out.toString().contains(searchString);
	}
}
