package de.fzj.unicore.xuudb;

/**
 * interface for checks on tokens.
 * This is useful for customising aspects of the XUUDB logic
 */
public interface ITokenChecker {

	/**
	 * compares two tokens for equivalence
	 * Example: compare X509 <-> X509
	 * or just the DNs
	 * 
	 * @param token1
	 * @param token2
	 * @return true if the tokens are the same
	 */
	public boolean sameToken(Object token1, Object token2);
	
	/**
	 * convert a token to some other form, stripping off unnecessary information
	 * Example: X509 Cert -> X500 Principal (DN)
	 * 
	 * @param token
	 * @return
	 */
	public Object stripToken(Object token);
	
	/**
	 * invoke checks on the token
	 * 
	 * @param token
	 * @return
	 */
	public boolean checkTokenValidity(Object token);
	
}
