/*********************************************************************************
 * Copyright (c) 2006 Forschungszentrum Juelich GmbH 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Forschungszentrum Juelich GmbH nor the names of its 
 * contributors may be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *********************************************************************************/
 

package de.fzj.unicore.xuudb;

import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

/**
 * does things on VM shutdown. Client code can register with this class to
 * do controlled shutdown operations.
 */
public class ShutdownHook extends Thread {
	private static boolean shutdownStat = false;
	private static int count = 0;
	private static Collection<IShutdownable> runners = null;
	
	private static ShutdownHook instance = null;
	public static ShutdownHook init() {
		if(instance==null)
			return instance = new ShutdownHook();
		else
			return instance;
	}
	
	private ShutdownHook() {
		runners = new Vector<IShutdownable>();
		Runtime.getRuntime().addShutdownHook(this);
	}

	public static void register(IShutdownable obj) {
		if(instance==null){
			instance = new ShutdownHook();
		}
		if(obj!=null)
			runners.add(obj);
	}
	
	public void run() {
		if (shutdownStat == false) { 
			shutdownStat = true;
			
			Iterator<IShutdownable> it = runners.iterator();
			while(it.hasNext()) {
				IShutdownable now = it.next();
				System.out.print("Shutting down <" + now.getNameOfService() + "> ...");
				try {
					now.shutdown();
				} catch (Exception e) {
					e.printStackTrace();
				} finally {
					System.out.println("Done");
				}
			}
			System.out.println("Bye.");
		} else {
			System.out.println("Please wait... " + count++);
			if (count == 10) {
				System.out.println("OK. Giving up... BYE");
				System.exit(0);
			}
		}
	}
}
