/*********************************************************************************
 * Copyright (c) 2006 Forschungszentrum Juelich GmbH 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Forschungszentrum Juelich GmbH nor the names of its 
 * contributors may be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *********************************************************************************/
 

package de.fzj.unicore.xuudb.client;

import java.io.File;
import java.net.URL;
import java.util.Properties;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.codehaus.xfire.client.XFireProxyFactory;
import org.codehaus.xfire.service.Service;
import org.codehaus.xfire.xmlbeans.XmlBeansServiceFactory;

import de.fzj.unicore.xuudb.Configuration;
import de.fzj.unicore.xuudb.interfaces.IAdmin;
import de.fzj.unicore.xuudb.interfaces.IPublic;

public class ServiceFactory extends XmlBeansServiceFactory {

    private static final Logger logger = Configuration.getLogger(ServiceFactory.class);

    private static ServiceFactory instance = null;
    private static Properties config;

    public static ServiceFactory getInstance(Properties c) {
	logger.info("Creating ServiceFactory..");
	if (instance == null) {
	    try {
		config = c;
		instance = new ServiceFactory();
	    } catch (Exception e) {
	    	logger.log(Level.ERROR,"Cannot create ServiceFactory",e);
	    	instance = null;
	    }
	}
	logger.info("    Done");
	return instance;
    }
	
	
    private Service serviceModel;
    private XFireProxyFactory proxyFactory;
    private IAdmin adminEndpoint;
    private IPublic publicEndpoint;
    private URL adminEndpointURL;
    private URL publicEndpointURL;
	
    private ServiceFactory() throws Exception {
	logger.info("Creating ServiceFactory ... ");
	proxyFactory = new XFireProxyFactory();
		
	if( Boolean.valueOf(config.getProperty(Configuration.USESSL)) ) {
	    logger.info("Configuring SSL");
	    File ks = new File(config.getProperty(Configuration.KEYSTORE));
	    if( !ks.exists() ) {
		logger.error("Keystore file does not exist: " + ks.getAbsolutePath());
		throw new Exception("Keystore file does not exist: " + ks.getAbsolutePath());
	    }
	    System.setProperty("javax.net.ssl.keyStore", ks.getAbsolutePath());
	    System.setProperty("javax.net.ssl.keyStoreType", config.getProperty(Configuration.KEYSTORETYPE,"JKS"));
	    System.setProperty("javax.net.ssl.keyStorePassword", config.getProperty(Configuration.KEYSTOREPASSWORD)); 

	    String tsName=config.getProperty(Configuration.TRUSTSTORE);
	    if( tsName==null || !new File(tsName).exists() ) {  // if truststore entry missing, user keystore as truststore (JKS)
		System.setProperty("javax.net.ssl.trustStore", ks.getAbsolutePath());
		System.setProperty("javax.net.ssl.trustStoreType", config.getProperty(Configuration.KEYSTORETYPE,"JKS"));
		System.setProperty("javax.net.ssl.trustStorePassword", config.getProperty(Configuration.KEYSTOREPASSWORD)); 
	    } else {
		File ts=new File(tsName);
		System.setProperty("javax.net.ssl.trustStore", ts.getAbsolutePath());
		System.setProperty("javax.net.ssl.trustStoreType", config.getProperty(Configuration.TRUSTSTORETYPE,"JKS"));
		System.setProperty("javax.net.ssl.trustStorePassword", config.getProperty(Configuration.TRUSTSTOREPASSWORD));
	    }
	}
	logger.info("Creating XUUDB URLs...");
	int port = Integer.parseInt(config.getProperty(Configuration.HTTPPORT));
	adminEndpointURL = new URL(config.getProperty(Configuration.HTTPHOST) + ":" + port + "/" + IAdmin.SERVICE_NAME);
	publicEndpointURL = new URL(config.getProperty(Configuration.HTTPHOST) + ":" + port + "/" + IPublic.SERVICE_NAME);
		
	logger.info("Creating XUUDB Admin Endpoint at " + adminEndpointURL.toString());
	serviceModel = this.create(IAdmin.class,IAdmin.SERVICE_NAME,null,null);
	adminEndpoint = (IAdmin) proxyFactory.create(serviceModel, adminEndpointURL.toString());
		
	serviceModel = this.create(IPublic.class,IPublic.SERVICE_NAME,null,null);
	logger.info("Creating XUUDB Query Endpoint at " + publicEndpointURL.toString());
	publicEndpoint = (IPublic) proxyFactory.create(serviceModel, publicEndpointURL.toString());
    }

    public IAdmin getAdminEndpoint() { return adminEndpoint; }
    public IPublic getPublicEndpoint() { return publicEndpoint; }

}
