/*********************************************************************************
 * Copyright (c) 2006 Forschungszentrum Juelich GmbH 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Forschungszentrum Juelich GmbH nor the names of its 
 * contributors may be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *********************************************************************************/
 

package de.fzj.unicore.xuudb.server;

import java.io.File;
import java.net.URI;
import java.util.Properties;

import org.apache.log4j.Logger;
import org.codehaus.xfire.XFire;
import org.codehaus.xfire.XFireFactory;
import org.codehaus.xfire.service.Service;
import org.codehaus.xfire.service.ServiceFactory;
import org.codehaus.xfire.service.invoker.BeanInvoker;
import org.codehaus.xfire.xmlbeans.XmlBeansServiceFactory;

import de.fzj.unicore.xuudb.Configuration;
import de.fzj.unicore.xuudb.IShutdownable;
import de.fzj.unicore.xuudb.ShutdownHook;
import de.fzj.unicore.xuudb.XUUDBInit;
import de.fzj.unicore.xuudb.interfaces.IAdmin;
import de.fzj.unicore.xuudb.interfaces.IPublic;

public class HttpsServer extends XUUDBInit implements IShutdownable {

	private static final Logger logger = Configuration.getLogger(HttpsServer.class);
	
	private JettyServer server;
	
	private XFire xfire;

	/**
	 * creates a XUUDB Http(s)Server like defined in the Properties prop
	 * @param p
	 * @throws Exception
	 */
	public HttpsServer(Properties p) throws Exception {
		super(p);
	}
	/**
	 * creates a XUUDB Http(s)Server like defined in the PropertiesFile cf
	 * @param cf
	 * @throws Exception
	 */
	public HttpsServer(String cf) throws Exception {
		super(cf);
	}	
	
	public void start() throws Exception {
		xfire = XFireFactory.newInstance().getXFire();
		ServiceFactory serviceFactory = new XmlBeansServiceFactory();

		// Create IAdmin Service
		Service adminService = serviceFactory.create(IAdmin.class,IAdmin.SERVICE_NAME,null,null);
		adminService.setInvoker(new BeanInvoker(new AdminImpl(XUUDBInit.getConfig())));
		String acl=XUUDBInit.getConfig().getProperty("xuudb_acl_file");
		if(acl!=null){
			adminService.addInHandler(new ACLHandler(new File(acl)));
		}
		else adminService.addInHandler(new ACLHandler());
		
		// Register the service in the ServiceRegistry
		xfire.getServiceRegistry().register(adminService);

		// Create IPublic Service
		Service publicService = serviceFactory.create(IPublic.class,IPublic.SERVICE_NAME,null,null);
		publicService.setInvoker(new BeanInvoker(new PublicImpl(XUUDBInit.getConfig())));
		// Register the service in the ServiceRegistry
		xfire.getServiceRegistry().register(publicService);

		
		// Start the HTTP/HTTPS server
		int port = Integer.valueOf(XUUDBInit.getConfig().getProperty(Configuration.HTTPPORT));
		String hostname=XUUDBInit.getConfig().getProperty(Configuration.HTTPHOST);
		String host=hostname!=null?URI.create(hostname).getHost():null;
		if(host==null){
			logger.info("No hostname specified, will listen on all network interfaces.");
		}
		if( Boolean.valueOf(XUUDBInit.getConfig().getProperty(Configuration.USESSL)) ) {
			logger.info("Setting up Jetty server in SSL mode.");
			server = new JettyServer(
					host,
					port,
					XUUDBInit.getConfig().getProperty(Configuration.KEYSTORE),
					XUUDBInit.getConfig().getProperty(Configuration.KEYSTORETYPE),
					XUUDBInit.getConfig().getProperty(Configuration.KEYSTOREPASSWORD),
					XUUDBInit.getConfig().getProperty(Configuration.TRUSTSTORE),
					XUUDBInit.getConfig().getProperty(Configuration.TRUSTSTORETYPE),
					XUUDBInit.getConfig().getProperty(Configuration.TRUSTSTOREPASSWORD) );
			server.start();
			System.out.println("Jetty SSL server started on port: " + port);
		} else {
			logger.info("Setting up Jetty server in non-SSL mode.");
			server = new JettyServer(host,port);
			server.start();
			System.out.println("Jetty non-SSL Server started on port: " + port);
		}
		//register with shutdownhook
		ShutdownHook.register(this);
		System.out.println("Send SIG or TERM or press Ctrl-c to shutdown gracefully.");
	}

	
	
	
	
	public void shutdown() throws Exception {
		logger.info("Shutting down Jetty server");
		server.stop();
	}

	public String getNameOfService() {
		return "XUUDB Server";
	}
}
